package com.chelvc.framework.database.support;

import java.util.Collection;

import com.chelvc.framework.database.listener.OperateListener;
import org.apache.commons.lang3.tuple.Pair;

/**
 * 数据操作事件业务接口
 *
 * @author Woody
 * @date 2024/4/30
 */
public interface EventService<T> {
    /**
     * 注册操作事件监听器
     *
     * @param listener 监听器实例
     */
    default void register(OperateListener<T> listener) {
    }

    /**
     * 发布数据新增事件
     *
     * @param entity 新增对象实例
     */
    default void publishAddition(T entity) {
    }

    /**
     * 批量发布数据新增事件
     *
     * @param entities 新增对象实例集合
     */
    default void publishAddition(Collection<T> entities) {
    }

    /**
     * 发布数据删除事件
     *
     * @param entity 删除对象实例
     */
    default void publishDeletion(T entity) {
    }

    /**
     * 批量发布数据删除事件
     *
     * @param entities 删除对象实例集合
     */
    default void publishDeletion(Collection<T> entities) {
    }

    /**
     * 发布数据更新事件
     *
     * @param before 更新前对象实例
     * @param after  更新后对象实例
     */
    default void publishModification(T before, T after) {
    }

    /**
     * 批量发布数据更新事件
     *
     * @param pairs 更新前后对象实例集合
     */
    default void publishModification(Collection<Pair<T, T>> pairs) {
    }
}
