package com.chelvc.framework.export;

import java.io.IOException;
import java.util.Collection;
import java.util.function.BiConsumer;
import java.util.function.BiFunction;
import javax.servlet.http.HttpServletResponse;

import com.chelvc.framework.common.util.ExcelUtils;
import lombok.NonNull;

/**
 * 数据导出操作接口
 *
 * @author Woody
 * @date 2024/5/9
 */
public interface ExportHandler {
    /**
     * 初始化处理器
     */
    default void initialize() {
    }

    /**
     * 获取导出任务状态
     *
     * @param id 导出任务ID
     * @return 导出任务状态
     */
    com.chelvc.framework.export.Exporting.Status status(long id);

    /**
     * 下载数据导出文件
     *
     * @param id       导出任务ID
     * @param response Http响应对象
     * @throws IOException I/O操作异常
     */
    void download(long id, HttpServletResponse response) throws IOException;

    /**
     * 异步导出数据
     *
     * @param filename 文件名称
     * @param provider 数据生产者
     * @param writer   Excel对象实例写入接口
     * @param titles   表头数组
     * @param <T>      数据类型
     * @return 导出任务ID
     */
    default <T> long export(@NonNull String filename, @NonNull BiFunction<Integer, Integer, Collection<T>> provider,
                            @NonNull ExcelUtils.Writer<T> writer, @NonNull String... titles) {
        return this.export(
                filename,
                (id, volume) -> {
                },
                provider, writer, titles
        );
    }

    /**
     * 异步导出数据
     *
     * @param filename 文件名称
     * @param listener 导出完成监听器
     * @param provider 数据生产者
     * @param writer   Excel对象实例写入接口
     * @param titles   表头数组
     * @param <T>      数据类型
     * @return 导出任务ID
     */
    <T> long export(String filename, BiConsumer<Long, Integer> listener,
                    BiFunction<Integer, Integer, Collection<T>> provider, ExcelUtils.Writer<T> writer,
                    String... titles);

    /**
     * 销毁处理器
     */
    default void destroy() {
    }
}
